@@ -637,6 +637,11 @@ class DaskExpr(
 
         return self._with_callable(da.exp)
 
+    def sin(self) -> Self:
+        import dask.array as da
+
+        return self._with_callable(da.sin)
+
     def sqrt(self) -> Self:
         import dask.array as da
 
@@ -693,6 +698,7 @@ class DaskExpr(
     def dt(self) -> DaskExprDateTimeNamespace:
         return DaskExprDateTimeNamespace(self)
 
+    any_value = not_implemented()
     filter = not_implemented()
     first = not_implemented()
     rank = not_implemented()
