@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import operator
 import warnings
-from typing import TYPE_CHECKING, Any, Callable, Literal, cast
+from typing import TYPE_CHECKING, Any, Callable, Literal
 
 import numpy as np
 
@@ -41,6 +41,7 @@ if TYPE_CHECKING:
     from narwhals._compliant.series import HistData
     from narwhals._pandas_like.dataframe import PandasLikeDataFrame
     from narwhals._pandas_like.namespace import PandasLikeNamespace
+    from narwhals._pandas_like.typing import NativeSeriesT
     from narwhals._typing import NoDefault
     from narwhals._utils import Version, _LimitedContext
     from narwhals.dtypes import DType
@@ -588,6 +589,10 @@ class PandasLikeSeries(EagerSeries[Any]):
         return res_ser
 
     def fill_nan(self, value: float | None) -> Self:
+        if self._implementation.is_cudf() and (value is None):
+            # TODO(Unassigned): https://github.com/narwhals-dev/narwhals/issues/3231
+            msg = "`fill_nan(value=None)` is not support for CuDF backend"
+            raise NotImplementedError(msg)
         if not self.dtype.is_numeric():  # pragma: no cover
             msg = f"`.fill_nan` only supported for numeric dtype and not {self.dtype}, did you mean `.fill_null`?"
             raise InvalidOperationError(msg)
@@ -815,58 +820,24 @@ class PandasLikeSeries(EagerSeries[Any]):
 
     def floor(self) -> Self:
         native = self.native
-        native_cls = type(native)
-        implementation = self._implementation
-        if get_dtype_backend(native.dtype, implementation=implementation) == "pyarrow":
+        if self.is_native_dtype_pyarrow(native.dtype):
             import pyarrow.compute as pc
 
-            from narwhals._arrow.utils import native_to_narwhals_dtype
-
-            ca = native.array._pa_array
-            result_arr = cast("ChunkedArrayAny", pc.floor(ca))
-            nw_dtype = native_to_narwhals_dtype(result_arr.type, self._version)
-            out_dtype = narwhals_to_native_dtype(
-                nw_dtype, "pyarrow", self._implementation, self._version
-            )
-            result_native = native_cls(
-                result_arr, dtype=out_dtype, index=native.index, name=native.name
-            )
+            result_native = self._apply_pyarrow_compute_func(native, pc.floor)
         else:
-            array_funcs = self._array_funcs
-            result_arr = array_funcs.floor(self.native)
-            result_native = (
-                native_cls(result_arr, index=native.index, name=native.name)
-                if implementation.is_cudf()
-                else result_arr
-            )
+            array_func = self._array_funcs.floor
+            result_native = self._apply_array_func(native, array_func)
         return self._with_native(result_native)
 
     def ceil(self) -> Self:
         native = self.native
-        native_cls = type(native)
-        implementation = self._implementation
-        if get_dtype_backend(native.dtype, implementation=implementation) == "pyarrow":
+        if self.is_native_dtype_pyarrow(native.dtype):
             import pyarrow.compute as pc
 
-            from narwhals._arrow.utils import native_to_narwhals_dtype
-
-            ca = native.array._pa_array
-            result_arr = cast("ChunkedArrayAny", pc.ceil(ca))
-            nw_dtype = native_to_narwhals_dtype(result_arr.type, self._version)
-            out_dtype = narwhals_to_native_dtype(
-                nw_dtype, "pyarrow", self._implementation, self._version
-            )
-            result_native = native_cls(
-                result_arr, dtype=out_dtype, index=native.index, name=native.name
-            )
+            result_native = self._apply_pyarrow_compute_func(native, pc.ceil)
         else:
-            array_funcs = self._array_funcs
-            result_arr = array_funcs.ceil(self.native)
-            result_native = (
-                native_cls(result_arr, index=native.index, name=native.name)
-                if implementation.is_cudf()
-                else result_arr
-            )
+            array_func = self._array_funcs.ceil
+            result_native = self._apply_array_func(native, array_func)
         return self._with_native(result_native)
 
     def to_dummies(self, *, separator: str, drop_first: bool) -> PandasLikeDataFrame:
@@ -1026,6 +997,13 @@ class PandasLikeSeries(EagerSeries[Any]):
         return self._with_native(result)
 
     def __iter__(self) -> Iterator[Any]:
+        if self._implementation.is_cudf():
+            msg = (
+                "Iterating over a cuDF Series, DataFrame or Index is not supported. "
+                "For more information see: https://docs.rapids.ai/api/cudf/stable/user_guide/pandas-comparison/#iteration"
+            )
+            raise NotImplementedError(msg)
+
         yield from self.native.__iter__()
 
     def __contains__(self, other: Any) -> bool:
@@ -1092,66 +1070,82 @@ class PandasLikeSeries(EagerSeries[Any]):
 
     def log(self, base: float) -> Self:
         native = self.native
-        native_cls = type(native)
-        implementation = self._implementation
-
-        if get_dtype_backend(native.dtype, implementation=implementation) == "pyarrow":
+        if self.is_native_dtype_pyarrow(native.dtype):
             import pyarrow.compute as pc
 
-            from narwhals._arrow.utils import native_to_narwhals_dtype
+            def pc_log(ca: ChunkedArrayAny) -> ChunkedArrayAny:
+                return pc.logb(ca, base)  # type: ignore[return-value]
 
-            ca = native.array._pa_array
-            result_arr = cast("ChunkedArrayAny", pc.logb(ca, base))
-            nw_dtype = native_to_narwhals_dtype(result_arr.type, self._version)
-            out_dtype = narwhals_to_native_dtype(
-                nw_dtype, "pyarrow", self._implementation, self._version
-            )
-            result_native = native_cls(
-                result_arr, dtype=out_dtype, index=native.index, name=native.name
-            )
+            result_native = self._apply_pyarrow_compute_func(native, pc_log)
         else:
-            array_funcs = self._array_funcs
-            result_arr = array_funcs.log(native) / array_funcs.log(base)
-            result_native = (
-                native_cls(result_arr, index=native.index, name=native.name)
-                if implementation.is_cudf()
-                else result_arr
-            )
+            log_func = self._array_funcs.log
 
+            def array_log(arr: NativeSeriesT) -> NativeSeriesT:
+                return log_func(arr) / log_func(base)  # pyright: ignore[reportArgumentType, reportCallIssue]
+
+            result_native = self._apply_array_func(native, array_log)
         return self._with_native(result_native)
 
     def exp(self) -> Self:
         native = self.native
-        native_cls = type(native)
-        implementation = self._implementation
-
-        if get_dtype_backend(native.dtype, implementation=implementation) == "pyarrow":
+        if self.is_native_dtype_pyarrow(native.dtype):
             import pyarrow.compute as pc
 
-            from narwhals._arrow.utils import native_to_narwhals_dtype
-
-            ca = native.array._pa_array
-            result_arr = cast("ChunkedArrayAny", pc.exp(ca))
-            nw_dtype = native_to_narwhals_dtype(result_arr.type, self._version)
-            out_dtype = narwhals_to_native_dtype(
-                nw_dtype, "pyarrow", self._implementation, self._version
-            )
-            result_native = native_cls(
-                result_arr, dtype=out_dtype, index=native.index, name=native.name
-            )
+            result_native = self._apply_pyarrow_compute_func(native, pc.exp)
         else:
-            result_arr = self._array_funcs.exp(native)
-            result_native = (
-                native_cls(result_arr, index=native.index, name=native.name)
-                if implementation.is_cudf()
-                else result_arr
-            )
+            array_func = self._array_funcs.exp
+            result_native = self._apply_array_func(native, array_func)
 
         return self._with_native(result_native)
 
     def sqrt(self) -> Self:
         return self._with_native(self.native.pow(0.5))
 
+    def sin(self) -> Self:
+        native = self.native
+        if self.is_native_dtype_pyarrow(native.dtype):
+            import pyarrow.compute as pc
+
+            result_native = self._apply_pyarrow_compute_func(
+                native,
+                pc.sin,  # type: ignore[arg-type]
+            )
+        else:
+            array_func = self._array_funcs.sin
+            result_native = self._apply_array_func(native, array_func)
+
+        return self._with_native(result_native)
+
+    def is_native_dtype_pyarrow(self, native_dtype: Any) -> bool:
+        impl = self._implementation
+        return get_dtype_backend(native_dtype, implementation=impl) == "pyarrow"
+
+    def _apply_pyarrow_compute_func(
+        self, native: NativeSeriesT, pc_func: Callable[[ChunkedArrayAny], ChunkedArrayAny]
+    ) -> NativeSeriesT:
+        from narwhals._arrow.utils import native_to_narwhals_dtype
+
+        native_cls = type(native)
+        result_arr = pc_func(native.array._pa_array)  # type: ignore[attr-defined]
+        nw_dtype = native_to_narwhals_dtype(result_arr.type, self._version)
+        out_dtype = narwhals_to_native_dtype(
+            nw_dtype, "pyarrow", self._implementation, self._version
+        )
+        return native_cls(
+            result_arr, dtype=out_dtype, index=native.index, name=native.name
+        )
+
+    def _apply_array_func(
+        self, native: NativeSeriesT, array_func: Callable[[NativeSeriesT], NativeSeriesT]
+    ) -> NativeSeriesT:
+        native_cls = type(native)
+        result_arr = array_func(native)
+        return (
+            native_cls(result_arr, index=native.index, name=native.name)
+            if self._implementation.is_cudf()
+            else result_arr
+        )
+
     def any_value(self, *, ignore_nulls: bool) -> PythonLiteral:
         return self.drop_nulls().first() if ignore_nulls else self.first()
 
