@@ -3,7 +3,11 @@ from __future__ import annotations
 from typing import TYPE_CHECKING, Any
 
 from narwhals._compliant.any_namespace import StringNamespace
-from narwhals._pandas_like.utils import PandasLikeSeriesNamespace, is_dtype_pyarrow
+from narwhals._pandas_like.utils import (
+    PandasLikeSeriesNamespace,
+    align_and_extract_native,
+    is_dtype_pyarrow,
+)
 
 if TYPE_CHECKING:
     from narwhals._pandas_like.series import PandasLikeSeries
@@ -16,21 +20,21 @@ class PandasLikeSeriesStringNamespace(
         return self.with_native(self.native.str.len())
 
     def replace(
-        self, pattern: str, value: str, *, literal: bool, n: int
+        self, value: PandasLikeSeries, pattern: str, *, literal: bool, n: int
     ) -> PandasLikeSeries:
-        try:
-            series = self.native.str.replace(
-                pat=pattern, repl=value, n=n, regex=not literal
-            )
-        except TypeError as e:
-            if not isinstance(value, str):
-                msg = f"{self.compliant._implementation} backed `.str.replace` only supports str replacement values"
-                raise TypeError(msg) from e
-            raise
+        _, value_native = align_and_extract_native(self.compliant, value)
+        if not isinstance(value_native, str):
+            msg = f"{self.compliant._implementation} backed `.str.replace` only supports str replacement values"
+            raise TypeError(msg)
+        series = self.native.str.replace(
+            pat=pattern, repl=value_native, n=n, regex=not literal
+        )
         return self.with_native(series)
 
-    def replace_all(self, pattern: str, value: str, *, literal: bool) -> PandasLikeSeries:
-        return self.replace(pattern, value, literal=literal, n=-1)
+    def replace_all(
+        self, value: PandasLikeSeries, pattern: str, *, literal: bool
+    ) -> PandasLikeSeries:
+        return self.replace(value, pattern, literal=literal, n=-1)
 
     def strip_chars(self, characters: str | None) -> PandasLikeSeries:
         return self.with_native(self.native.str.strip(characters))
@@ -88,5 +92,8 @@ class PandasLikeSeriesStringNamespace(
     def to_lowercase(self) -> PandasLikeSeries:
         return self.with_native(self.native.str.lower())
 
+    def to_titlecase(self) -> PandasLikeSeries:
+        return self.with_native(self.native.str.title())
+
     def zfill(self, width: int) -> PandasLikeSeries:
         return self.with_native(self.native.str.zfill(width))
